@@ -46,6 +46,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <string.h>
 #include "nautilus-desktop-window.h"
 #include "nautilus-first-time-druid.h"
 #include "nautilus-main.h"
@@ -54,37 +55,26 @@
 #include "nautilus-shell-interface.h"
 #include "nautilus-shell.h"
 #include "nautilus-window-bookmarks.h"
+#include "libnautilus-private/nautilus-file-operations.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-manage-views.h"
 #include <libxml/xmlsave.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
-#include <eel/eel-string-list.h>
-#include <eel/eel-string.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtkinvisible.h>
-#include <gtk/gtksignal.h>
 #include <gtk/gtkwindow.h>
 #include <libgnome/gnome-config.h>
-#include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-authentication-manager.h>
 #include <libgnomeui/gnome-client.h>
-#include <libgnomeui/gnome-messagebox.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
-#include <libnautilus-private/nautilus-metafile-factory.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-private/nautilus-desktop-link-monitor.h>
@@ -92,6 +82,8 @@
 #include <libnautilus-private/nautilus-signaller.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
 #include <bonobo-activation/bonobo-activation.h>
+#include <libnautilus-private/nautilus-autorun.h>
+
 #ifdef HAVE_STARTUP_NOTIFICATION
 #define SN_API_NOT_YET_FROZEN Yes_i_know_DO_IT
 #include <libsn/sn-launchee.h>
@@ -117,12 +109,20 @@ static GList *nautilus_application_spatial_window_list;
 
 static void     desktop_changed_callback          (gpointer                  user_data);
 static void     desktop_location_changed_callback (gpointer                  user_data);
-static void     volume_unmounted_callback         (GnomeVFSVolumeMonitor    *monitor,
-						   GnomeVFSVolume           *volume,
-						   NautilusApplication      *application);
-static void     volume_mounted_callback           (GnomeVFSVolumeMonitor    *monitor,
-						   GnomeVFSVolume           *volume,
-						   NautilusApplication      *application);
+static void     mount_removed_callback            (GVolumeMonitor            *monitor,
+						   GMount                    *mount,
+						   NautilusApplication       *application);
+static void     mount_added_callback              (GVolumeMonitor            *monitor,
+						   GMount                    *mount,
+						   NautilusApplication       *application);
+static void     volume_added_callback              (GVolumeMonitor           *monitor,
+						    GVolume                  *volume,
+						    NautilusApplication      *application);
+static void     drive_connected_callback           (GVolumeMonitor           *monitor,
+						    GDrive                   *drive,
+						    NautilusApplication      *application);
+static void     drive_listen_for_eject_button      (GDrive *drive, 
+						    NautilusApplication *application);
 static void     update_session                    (gpointer                  callback_data);
 static void     init_session                      (void);
 static gboolean is_kdesktop_present               (void);
@@ -143,8 +143,6 @@ create_object (PortableServer_Servant servant,
 	if (strcmp (iid, SHELL_IID) == 0) {
 		application = NAUTILUS_APPLICATION (bonobo_object_from_servant (servant));
 		object = BONOBO_OBJECT (nautilus_shell_new (application));
-	} else if (strcmp (iid, METAFILE_FACTORY_IID) == 0) {
-		object = BONOBO_OBJECT (nautilus_metafile_factory_get_instance ());
 	} else {
 		object = CORBA_OBJECT_NIL;
 	}
@@ -171,6 +169,48 @@ nautilus_application_get_n_windows (void)
 	       g_list_length (nautilus_application_desktop_windows);
 }
 
+static void
+startup_volume_mount_cb (GObject *source_object,
+			 GAsyncResult *res,
+			 gpointer user_data)
+{
+	if (g_volume_mount_finish (G_VOLUME (source_object), res, NULL)) {
+		nautilus_inhibit_autorun_for_volume (G_VOLUME (source_object));
+	}
+}
+
+static void
+automount_all_volumes (NautilusApplication *application)
+{
+	GList *volumes, *l;
+	GMount *mount;
+	GVolume *volume;
+		
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT)) {
+		/* automount all mountable volumes at start-up */
+		volumes = g_volume_monitor_get_volumes (application->volume_monitor);
+		for (l = volumes; l != NULL; l = l->next) {
+			volume = l->data;
+			
+			if (!g_volume_should_automount (volume) ||
+			    !g_volume_can_mount (volume)) {
+				continue;
+			}
+			
+			mount = g_volume_get_mount (volume);
+			if (mount != NULL) {
+				g_object_unref (mount);
+				continue;
+			}
+
+			/* pass NULL as GMountOperation to avoid user interaction */
+			g_volume_mount (volume, 0, NULL, NULL, startup_volume_mount_cb, NULL);
+		}
+		eel_g_object_list_free (volumes);
+	}
+	
+}
+
 static void
 nautilus_application_instance_init (NautilusApplication *application)
 {
@@ -179,18 +219,6 @@ nautilus_application_instance_init (NautilusApplication *application)
 
 	application->shell = nautilus_shell_new (application);
 	
-	/* Watch for volume mounts so we can restore open windows
-	 * This used to be for showing new window on mount, but is not
-	 * used anymore */
-
-	/* Watch for volume unmounts so we can close open windows */
-	g_signal_connect_object (gnome_vfs_get_volume_monitor (), "volume_unmounted",
-				 G_CALLBACK (volume_unmounted_callback), application, 0);
-	g_signal_connect_object (gnome_vfs_get_volume_monitor (), "volume_pre_unmount",
-				 G_CALLBACK (volume_unmounted_callback), application, 0);
-	g_signal_connect_object (gnome_vfs_get_volume_monitor (), "volume_mounted",
-				 G_CALLBACK (volume_mounted_callback), application, 0);
-
 	/* register views */
 	fm_icon_view_register ();
 	fm_desktop_icon_view_register ();
@@ -236,11 +264,21 @@ nautilus_application_destroy (BonoboObject *object)
 	
 	g_object_unref (application->undo_manager);
 
+	if (application->volume_monitor) {
+		g_object_unref (application->volume_monitor);
+		application->volume_monitor = NULL;
+	}
+	
 	if (application->shell_registered) {
 		bonobo_activation_unregister_active_server (SHELL_IID, BONOBO_OBJREF (application->shell));
 	}
 	bonobo_object_unref (application->shell);
 
+	if (application->automount_idle_id != 0) {
+		g_source_remove (application->automount_idle_id);
+		application->automount_idle_id = 0;
+	}
+
 	EEL_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
 }
 
@@ -249,58 +287,68 @@ check_required_directories (NautilusApplication *application)
 {
 	char *user_directory;
 	char *desktop_directory;
-	EelStringList *directories;
-	char *directories_as_string;
-	char *error_string;
-	char *detail_string;
-	GtkDialog *dialog;
-	int failed_count;
-	
+	GSList *directories;
+	gboolean ret;
+
 	g_assert (NAUTILUS_IS_APPLICATION (application));
 
+	ret = TRUE;
+
 	user_directory = nautilus_get_user_directory ();
 	desktop_directory = nautilus_get_desktop_directory ();
 
-	directories = eel_string_list_new (TRUE);
-	
+	directories = NULL;
+
 	if (!g_file_test (user_directory, G_FILE_TEST_IS_DIR)) {
-		eel_string_list_insert (directories, user_directory);
+		directories = g_slist_prepend (directories, user_directory);
 	}
-	g_free (user_directory);	    
-	    
+
 	if (!g_file_test (desktop_directory, G_FILE_TEST_IS_DIR)) {
-		eel_string_list_insert (directories, desktop_directory);
+		directories = g_slist_prepend (directories, desktop_directory);
 	}
-	g_free (desktop_directory);
 
-	failed_count = eel_string_list_get_length (directories);
+	if (directories != NULL) {
+		int failed_count;
+		GString *directories_as_string;
+		GSList *l;
+		char *error_string;
+		const char *detail_string;
+		GtkDialog *dialog;
 
-	if (failed_count != 0) {
-		directories_as_string = eel_string_list_as_string (directories, ", ", EEL_STRING_LIST_ALL_STRINGS);
+		ret = FALSE;
+
+		failed_count = g_slist_length (directories);
+
+		directories_as_string = g_string_new ((const char *)directories->data);
+		for (l = directories->next; l != NULL; l = l->next) {
+			g_string_append_printf (directories_as_string, ", %s", (const char *)l->data);
+		}
 
 		if (failed_count == 1) {
 			error_string = g_strdup_printf (_("Nautilus could not create the required folder \"%s\"."),
-							directories_as_string);
+							directories_as_string->str);
 			detail_string = _("Before running Nautilus, please create the following folder, or "
 					  "set permissions such that Nautilus can create it.");
 		} else {
 			error_string = g_strdup_printf (_("Nautilus could not create the following required folders: "
-							  "%s."), directories_as_string);
-  			detail_string = _("Before running Nautilus, please create these folders, or "
+							  "%s."), directories_as_string->str);
+			detail_string = _("Before running Nautilus, please create these folders, or "
 					  "set permissions such that Nautilus can create them.");
 		}
-		
+
 		dialog = eel_show_error_dialog (error_string, detail_string, NULL);
 		/* We need the main event loop so the user has a chance to see the dialog. */
 		nautilus_main_event_loop_register (GTK_OBJECT (dialog));
 
-		g_free (directories_as_string);
+		g_string_free (directories_as_string, TRUE);
 		g_free (error_string);
 	}
 
-	eel_string_list_free (directories);
+	g_slist_free (directories);
+	g_free (user_directory);
+	g_free (desktop_directory);
 
-	return failed_count == 0;
+	return ret;
 }
 
 static Nautilus_URIList *
@@ -308,6 +356,7 @@ nautilus_make_uri_list_from_shell_strv (const char * const *strv)
 {
 	int length, i;
 	Nautilus_URIList *uri_list;
+	GFile *file;
 	char *translated_uri;
 
 	length = g_strv_length ((char **) strv);
@@ -317,7 +366,9 @@ nautilus_make_uri_list_from_shell_strv (const char * const *strv)
 	uri_list->_length = length;
 	uri_list->_buffer = CORBA_sequence_Nautilus_URI_allocbuf (length);
 	for (i = 0; i < length; i++) {
-		translated_uri = gnome_vfs_make_uri_from_shell_arg (strv[i]);
+		file = g_file_new_for_commandline_arg (strv[i]);
+		translated_uri = g_file_get_uri (file);
+		g_object_unref (file);
 		uri_list->_buffer[i] = CORBA_string_dup (translated_uri);
 		g_free (translated_uri);
 		translated_uri = NULL;
@@ -327,49 +378,6 @@ nautilus_make_uri_list_from_shell_strv (const char * const *strv)
 	return uri_list;
 }
 
-static void
-migrate_old_nautilus_files (void)
-{
-	char *new_desktop_dir;
-	char *old_desktop_dir;
-	char *migrated_file;
-	char *link_name;
-	char *link_path;
-	int fd;
-	
-	old_desktop_dir = nautilus_get_gmc_desktop_directory ();
-	if (!g_file_test (old_desktop_dir, G_FILE_TEST_IS_DIR) ||
-	    g_file_test (old_desktop_dir, G_FILE_TEST_IS_SYMLINK)) {
-		g_free (old_desktop_dir);
-		return;
-	}
-	migrated_file = g_build_filename (old_desktop_dir, ".migrated", NULL);
-	if (!g_file_test (migrated_file, G_FILE_TEST_EXISTS)) {
-		link_name = g_filename_from_utf8 (_("Link To Old Desktop"), -1, NULL, NULL, NULL);
-		new_desktop_dir = nautilus_get_desktop_directory ();
-		link_path = g_build_filename (new_desktop_dir, link_name, NULL);
-	
-		
-		symlink ("../.gnome-desktop", link_path);
-		
-		g_free (link_name);
-		g_free (new_desktop_dir);
-		g_free (link_path);
-
-		fd = creat (migrated_file, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
-		if (fd >= 0) {
-			close (fd);
-		}
-		
-		eel_show_info_dialog (_("A link called \"Link To Old Desktop\" has been created on the desktop."),
-				      _("The location of the desktop directory has changed in GNOME 2.4. "
-					"You can open the link and move over the files you want, then delete the link."),
-				      NULL);
-	}
-	g_free (old_desktop_dir);
-	g_free (migrated_file);
-}
-
 static void
 menu_provider_items_updated_handler (NautilusMenuProvider *provider, GtkWidget* parent_window, gpointer data)
 {
@@ -399,9 +407,22 @@ menu_provider_init_callback (void)
         nautilus_module_extension_list_free (providers);
 }
 
+static gboolean
+automount_all_volumes_idle_cb (gpointer data)
+{
+	NautilusApplication *application = NAUTILUS_APPLICATION (data);
+
+	automount_all_volumes (application);
+
+	application->automount_idle_id = 0;
+	return FALSE;
+}
+
 static void
 finish_startup (NautilusApplication *application)
 {
+	GList *drives;
+
 	/* initialize nautilus modules */
 	nautilus_module_init ();
 
@@ -410,14 +431,37 @@ finish_startup (NautilusApplication *application)
 	/* attach menu-provider module callback */
 	menu_provider_init_callback ();
 	
-	/* initialize URI authentication manager */
-	gnome_authentication_manager_init ();
-
-	/* Make the desktop work with old Nautilus. */
-	migrate_old_nautilus_files ();
-
 	/* Initialize the desktop link monitor singleton */
 	nautilus_desktop_link_monitor_get ();
+
+	/* Watch for mounts so we can restore open windows This used
+	 * to be for showing new window on mount, but is not used
+	 * anymore */
+
+	/* Watch for unmounts so we can close open windows */
+	/* TODO-gio: This should be using the UNMOUNTED feature of GFileMonitor instead */
+	application->volume_monitor = g_volume_monitor_get ();
+	g_signal_connect_object (application->volume_monitor, "mount_removed",
+				 G_CALLBACK (mount_removed_callback), application, 0);
+	g_signal_connect_object (application->volume_monitor, "mount_pre_unmount",
+				 G_CALLBACK (mount_removed_callback), application, 0);
+	g_signal_connect_object (application->volume_monitor, "mount_added",
+				 G_CALLBACK (mount_added_callback), application, 0);
+	g_signal_connect_object (application->volume_monitor, "volume_added",
+				 G_CALLBACK (volume_added_callback), application, 0);
+	g_signal_connect_object (application->volume_monitor, "drive_connected",
+				 G_CALLBACK (drive_connected_callback), application, 0);
+
+	/* listen for eject button presses */
+	drives = g_volume_monitor_get_connected_drives (application->volume_monitor);
+	g_list_foreach (drives, (GFunc) drive_listen_for_eject_button, application);
+	g_list_foreach (drives, (GFunc) g_object_unref, NULL);
+	g_list_free (drives);
+
+	application->automount_idle_id = 
+		g_idle_add_full (G_PRIORITY_LOW,
+				 automount_all_volumes_idle_cb,
+				 application, NULL);
 }
 
 static void
@@ -433,7 +477,7 @@ initialize_kde_trash_hack (void)
 	trash_dir = NULL;
 
 	desktop_uri = nautilus_get_desktop_directory_uri_no_create ();
-	desktop_dir = gnome_vfs_get_local_path_from_uri (desktop_uri);
+	desktop_dir = g_filename_from_uri (desktop_uri, NULL, NULL);
 	g_free (desktop_uri);
 	
 	if (g_file_test (desktop_dir, G_FILE_TEST_EXISTS)) {
@@ -703,14 +747,13 @@ selection_get_cb (GtkWidget          *widget,
 static GtkWidget *
 get_desktop_manager_selection (GdkDisplay *display, int screen)
 {
-	char *selection_name;
+	char selection_name[32];
 	GdkAtom selection_atom;
 	Window selection_owner;
 	GtkWidget *selection_widget;
 
-	selection_name = g_strdup_printf ("_NET_DESKTOP_MANAGER_S%d", screen);
+	g_snprintf (selection_name, sizeof (selection_name), "_NET_DESKTOP_MANAGER_S%d", screen);
 	selection_atom = gdk_atom_intern (selection_name, FALSE);
-	g_free (selection_name);
 
 	selection_owner = XGetSelectionOwner (GDK_DISPLAY_XDISPLAY (display),
 					      gdk_x11_atom_to_xatom_for_display (display, 
@@ -851,21 +894,22 @@ nautilus_application_close_all_navigation_windows (void)
 }
 
 static NautilusSpatialWindow *
-nautilus_application_get_existing_spatial_window (const char *location)
+nautilus_application_get_existing_spatial_window (GFile *location)
 {
 	GList *l;
-	
+
 	for (l = nautilus_application_get_spatial_window_list ();
 	     l != NULL; l = l->next) {
-		char *window_location;
-		
+		GFile *window_location;
+
 		window_location = nautilus_window_get_location (NAUTILUS_WINDOW (l->data));
-		if (window_location != NULL &&
-		    strcmp (location, window_location) == 0) {
-			g_free (window_location);
-			return NAUTILUS_SPATIAL_WINDOW (l->data);
+		if (window_location != NULL) {
+			if (g_file_equal (location, window_location)) {
+				g_object_unref (window_location);
+				return NAUTILUS_SPATIAL_WINDOW (l->data);
+			}
+			g_object_unref (window_location);
 		}
-		g_free (window_location);
 	}
 	return NULL;
 }
@@ -875,41 +919,36 @@ find_parent_spatial_window (NautilusSpatialWindow *window)
 {
 	NautilusFile *file;
 	NautilusFile *parent_file;
-	char *location;
-	char *desktop_directory;
+	GFile *location;
 
 	location = nautilus_window_get_location (NAUTILUS_WINDOW (window));
 	if (location == NULL) {
 		return NULL;
 	}
 	file = nautilus_file_get (location);
-	g_free (location);
+	g_object_unref (location);
 
 	if (!file) {
 		return NULL;
 	}
-	
-	desktop_directory = nautilus_get_desktop_directory_uri ();	
-	
+
 	parent_file = nautilus_file_get_parent (file);
 	nautilus_file_unref (file);
 	while (parent_file) {
 		NautilusSpatialWindow *parent_window;
-		
-		location = nautilus_file_get_uri (parent_file);
 
-		/* Stop at the desktop directory, as this is the
-		 * conceptual root of the spatial windows */
-		if (!strcmp (location, desktop_directory)) {
-			g_free (location);
-			g_free (desktop_directory);
+		location = nautilus_file_get_location (parent_file);
+		parent_window = nautilus_application_get_existing_spatial_window (location);
+		g_object_unref (location);
+
+		/* Stop at the desktop directory if it's not explicitely opened
+		 * in a spatial window of its own.
+		 */
+		if (nautilus_file_is_desktop_directory (parent_file) && !parent_window) {
 			nautilus_file_unref (parent_file);
 			return NULL;
 		}
 
-		parent_window = nautilus_application_get_existing_spatial_window (location);
-		g_free (location);
-		
 		if (parent_window) {
 			nautilus_file_unref (parent_file);
 			return parent_window;
@@ -918,7 +957,6 @@ find_parent_spatial_window (NautilusSpatialWindow *window)
 		parent_file = nautilus_file_get_parent (file);
 		nautilus_file_unref (file);
 	}
-	g_free (desktop_directory);
 
 	return NULL;
 }
@@ -1025,7 +1063,7 @@ NautilusWindow *
 nautilus_application_present_spatial_window (NautilusApplication *application,
 					     NautilusWindow      *requesting_window,
 					     const char          *startup_id,
-					     const char          *location,
+					     GFile               *location,
 					     GdkScreen           *screen)
 {
 	return nautilus_application_present_spatial_window_with_selection (application,
@@ -1117,19 +1155,20 @@ NautilusWindow *
 nautilus_application_present_spatial_window_with_selection (NautilusApplication *application,
 							    NautilusWindow      *requesting_window,
 							    const char          *startup_id,
-							    const char          *location,
+							    GFile               *location,
 							    GList		*new_selection,
 							    GdkScreen           *screen)
 {
 	NautilusWindow *window;
 	GList *l;
+	char *uri;
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
-
+	
 	for (l = nautilus_application_get_spatial_window_list ();
 	     l != NULL; l = l->next) {
 		NautilusWindow *existing_window;
-		char *existing_location;
+		GFile *existing_location;
                	     
 		existing_window = NAUTILUS_WINDOW (l->data);
 		existing_location = existing_window->details->pending_location;
@@ -1137,8 +1176,8 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 		if (existing_location == NULL) {
 			existing_location = existing_window->details->location;
 		}
-
-		if (gnome_vfs_uris_match (existing_location, location)) {
+		
+		if (g_file_equal (existing_location, location)) {
 #ifdef HAVE_STARTUP_NOTIFICATION
 			end_startup_notification (GTK_WIDGET (existing_window),
 						  startup_id);
@@ -1150,9 +1189,11 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 				nautilus_view_set_selection (existing_window->content_view, new_selection);
 			}
 
+			uri = g_file_get_uri (location);
 			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
 					    "present EXISTING spatial window=%p: %s",
-					    existing_window, location);
+					    existing_window, uri);
+			g_free (uri);
 			return existing_window;
 		}
 	}
@@ -1191,9 +1232,11 @@ nautilus_application_present_spatial_window_with_selection (NautilusApplication
 	
 	nautilus_window_go_to_with_selection (window, location, new_selection);
 
+	uri = g_file_get_uri (location);
 	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
 			    "present NEW spatial window=%p: %s",
-			    window, location);
+			    window, uri);
+	g_free (uri);
 	
 	return window;
 }
@@ -1298,72 +1341,146 @@ window_can_be_closed (NautilusWindow *window)
 }
 
 static void
-volume_mounted_callback (GnomeVFSVolumeMonitor *monitor,
-			 GnomeVFSVolume *volume,
-			 NautilusApplication *application)
+volume_added_callback (GVolumeMonitor *monitor,
+		       GVolume *volume,
+		       NautilusApplication *application)
+{
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT) &&
+	    g_volume_should_automount (volume) &&
+	    g_volume_can_mount (volume)) {
+		nautilus_file_operations_mount_volume (NULL, volume, FALSE);
+	}
+}
+
+static void
+drive_eject_cb (GObject *source_object,
+		GAsyncResult *res,
+		gpointer user_data)
+{
+	GError *error;
+	char *primary;
+	char *name;
+	error = NULL;
+	if (!g_drive_eject_finish (G_DRIVE (source_object), res, &error)) {
+		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
+			name = g_drive_get_name (G_DRIVE (source_object));
+			primary = g_strdup_printf (_("Unable to eject %s"), name);
+			g_free (name);
+			eel_show_error_dialog (primary,
+					       error->message,
+				       NULL);
+			g_free (primary);
+		}
+		g_error_free (error);
+	}
+}
+
+static void
+drive_eject_button_pressed (GDrive *drive,
+			    NautilusApplication *application)
+{
+	g_drive_eject (drive, 0, NULL, drive_eject_cb, NULL);
+}
+
+static void
+drive_listen_for_eject_button (GDrive *drive, NautilusApplication *application)
+{
+	g_signal_connect (drive,
+			  "eject-button",
+			  G_CALLBACK (drive_eject_button_pressed),
+			  application);
+}
+
+static void
+drive_connected_callback (GVolumeMonitor *monitor,
+			  GDrive *drive,
+			  NautilusApplication *application)
+{
+	drive_listen_for_eject_button (drive, application);
+}
+
+static void
+autorun_show_window (GMount *mount, gpointer user_data)
+{
+	GFile *location;
+	NautilusApplication *application = user_data;
+	
+	location = g_mount_get_root (mount);
+	
+	/* Ther should probably be an easier way to do this */
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+		NautilusWindow *window;
+		window = nautilus_application_create_navigation_window (application, 
+									NULL, 
+									gdk_screen_get_default ());
+		nautilus_window_go_to (window, location);
+
+	} else {
+		nautilus_application_present_spatial_window (application, 
+							     NULL, 
+							     NULL, 
+							     location, 
+							     gdk_screen_get_default ());
+	}
+	g_object_unref (location);
+}
+
+static void
+mount_added_callback (GVolumeMonitor *monitor,
+		      GMount *mount,
+		      NautilusApplication *application)
 {
-	char *activation_uri;
 	NautilusDirectory *directory;
+	GFile *root;
 		
-	activation_uri = gnome_vfs_volume_get_activation_uri (volume);
-	directory = nautilus_directory_get_existing (activation_uri);
-	g_free (activation_uri);
+	root = g_mount_get_root (mount);
+	directory = nautilus_directory_get_existing (root);
+	g_object_unref (root);
 	if (directory != NULL) {
 		nautilus_directory_force_reload (directory);
 		nautilus_directory_unref (directory);
 	}
+
+	nautilus_autorun (mount, autorun_show_window, application);
 }
 
-/* Called whenever a volume is unmounted. Check and see if there are any windows open
- * displaying contents on the volume. If there are, close them.
- * It would also be cool to save open window and position info.
+/* Called whenever a mount is unmounted. Check and see if there are
+ * any windows open displaying contents on the mount. If there are,
+ * close them.  It would also be cool to save open window and position
+ * info.
  *
  * This is also called on pre_unmount.
  */
 static void
-volume_unmounted_callback (GnomeVFSVolumeMonitor *monitor,
-			   GnomeVFSVolume *volume,
-			   NautilusApplication *application)
+mount_removed_callback (GVolumeMonitor *monitor,
+			GMount *mount,
+			NautilusApplication *application)
 {
 	GList *window_list, *node, *close_list;
 	NautilusWindow *window;
-	char *uri, *activation_uri, *path;
-	GnomeVFSVolumeMonitor *volume_monitor;
-	GnomeVFSVolume *window_volume;
-	
+	GFile *root;
+
 	close_list = NULL;
 	
-	/* Check and see if any of the open windows are displaying contents from the unmounted volume */
+	/* Check and see if any of the open windows are displaying contents from the unmounted mount */
 	window_list = nautilus_application_get_window_list ();
 
-	volume_monitor = gnome_vfs_get_volume_monitor ();
-
-	activation_uri = gnome_vfs_volume_get_activation_uri (volume);
+	root = g_mount_get_root (mount);
 	/* Construct a list of windows to be closed. Do not add the non-closable windows to the list. */
 	for (node = window_list; node != NULL; node = node->next) {
 		window = NAUTILUS_WINDOW (node->data);
 		if (window != NULL && window_can_be_closed (window)) {
-			uri = nautilus_window_get_location (window);
-			if (eel_str_has_prefix (uri, activation_uri)) {
+			GFile *location;
+
+			location = nautilus_window_get_location (window);
+
+			if (g_file_has_prefix (location, root)) {
 				close_list = g_list_prepend (close_list, window);
-			} else {
-				path = gnome_vfs_get_local_path_from_uri (uri);
-				if (path != NULL) {
-					window_volume = gnome_vfs_volume_monitor_get_volume_for_path (volume_monitor,
-												      path);
-					if (window_volume != NULL && window_volume == volume) {
-						close_list = g_list_prepend (close_list, window);
-					}
-					gnome_vfs_volume_unref (window_volume);
-					g_free (path);
-				}
-				
-			}
-			g_free (uri);
+			} 
+			g_object_unref (location);
 		}
 	}
-	g_free (activation_uri);
-		
+
 	/* Handle the windows in the close list. */
 	for (node = close_list; node != NULL; node = node->next) {
 		window = NAUTILUS_WINDOW (node->data);
@@ -1373,17 +1490,55 @@ volume_unmounted_callback (GnomeVFSVolumeMonitor *monitor,
 			nautilus_window_go_home (window);
 		}
 	}
-		
+
 	g_list_free (close_list);
 }
 
-
 static void
 removed_from_session (GnomeClient *client, gpointer data)
 {
 	nautilus_main_event_loop_quit (FALSE);
 }
 
+static char *
+icon_to_string (GIcon *icon)
+{
+	const char * const *names;
+	GFile *file;
+	
+	if (icon == NULL) {
+		return NULL;
+	} else if (G_IS_THEMED_ICON (icon)) {
+		names = g_themed_icon_get_names (G_THEMED_ICON (icon));
+		return g_strjoinv (":", (char **)names);		
+	} else if (G_IS_FILE_ICON (icon)) {
+		file = g_file_icon_get_file (G_FILE_ICON (icon));
+		return g_file_get_path (file);
+	}
+	return NULL;
+}
+
+static GIcon *
+icon_from_string (const char *string)
+{
+	GFile *file;
+	GIcon *icon;
+	gchar **names;
+	
+	if (g_path_is_absolute (string)) {
+		file = g_file_new_for_path (string);
+		icon = g_file_icon_new (file);
+		g_object_unref (file);
+		return icon;
+	} else {
+		names = g_strsplit (string, ":", 0);
+		icon = g_themed_icon_new_from_names (names, -1);
+		g_strfreev (names);
+		return icon;
+	}
+	return NULL;
+}
+
 static char *
 save_session_to_file (void)
 {
@@ -1406,6 +1561,7 @@ save_session_to_file (void)
 	for (l = nautilus_get_history_list (); l != NULL; l = l->next) {
 		NautilusBookmark *bookmark;
 		xmlNodePtr bookmark_node;
+		GIcon *icon;
 		char *tmp;
 
 		bookmark = l->data;
@@ -1416,9 +1572,13 @@ save_session_to_file (void)
 		xmlNewProp (bookmark_node, "name", tmp);
 		g_free (tmp);
 
-		tmp = nautilus_bookmark_get_icon (bookmark);
-		xmlNewProp (bookmark_node, "icon", tmp);
-		g_free (tmp);
+		icon = nautilus_bookmark_get_icon (bookmark);
+		tmp = icon_to_string (icon);
+		g_object_unref (icon);
+		if (tmp) {
+			xmlNewProp (bookmark_node, "icon", tmp);
+			g_free (tmp);
+		}
 
 		tmp = nautilus_bookmark_get_uri (bookmark);
 		xmlNewProp (bookmark_node, "uri", tmp);
@@ -1453,9 +1613,19 @@ save_session_to_file (void)
 			    gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_MAXIMIZED) {
 				xmlNewProp (win_node, "maximized", "TRUE");
 			}
+
+			if (GTK_WIDGET (window)->window &&
+			    gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_STICKY) {
+				xmlNewProp (win_node, "sticky", "TRUE");
+			}
+
+			if (GTK_WIDGET (window)->window &&
+			    gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_ABOVE) {
+				xmlNewProp (win_node, "keep-above", "TRUE");
+			}
 		}
 
-		tmp = nautilus_window_get_location (window);
+		tmp = nautilus_window_get_location_uri (window);
 		xmlNewProp (win_node, "location", tmp);
 		g_free (tmp);
 	}
@@ -1524,19 +1694,31 @@ nautilus_application_load_session (NautilusApplication *application,
 						if (!strcmp (bookmark_node->name, "text")) {
 							continue;
 						} else if (!strcmp (bookmark_node->name, "bookmark")) {
-							xmlChar *name, *icon, *uri;
+							xmlChar *name, *icon_str, *uri;
 							gboolean has_custom_name;
+							GIcon *icon;
+							GFile *location;
 
 							uri = xmlGetProp (bookmark_node, "uri");
 							name = xmlGetProp (bookmark_node, "name");
 							has_custom_name = xmlHasProp (bookmark_node, "has_custom_name") ? TRUE : FALSE;
-							icon = xmlGetProp (bookmark_node, "icon");
+							icon_str = xmlGetProp (bookmark_node, "icon");
+							icon = NULL;
+							if (icon_str) {
+								icon = icon_from_string (icon_str);
+							}
+							location = g_file_new_for_uri (uri);
 
-							emit_change |= nautilus_add_to_history_list_no_notify (uri, name, has_custom_name, icon);
+							emit_change |= nautilus_add_to_history_list_no_notify (location, name, has_custom_name, icon);
 
+							g_object_unref (location);
+
+							if (icon) {
+								g_object_unref (icon);
+							}
 							xmlFree (name);
 							xmlFree (uri);
-							xmlFree (icon);
+							xmlFree (icon_str);
 						} else {
 							g_message ("unexpected bookmark node %s while parsing %s", bookmark_node->name, filename);
 							bail = TRUE;
@@ -1549,7 +1731,8 @@ nautilus_application_load_session (NautilusApplication *application,
 					}
 				} else if (!strcmp (node->name, "window")) {
 					NautilusWindow *window;
-					xmlChar *type, *location;
+					xmlChar *type, *location_uri;
+					GFile *location;
 
 					type = xmlGetProp (node, "type");
 					if (type == NULL) {
@@ -1558,8 +1741,8 @@ nautilus_application_load_session (NautilusApplication *application,
 						continue;
 					}
 
-					location = xmlGetProp (node, "location");
-					if (location == NULL) {
+					location_uri = xmlGetProp (node, "location");
+					if (location_uri == NULL) {
 						g_message ("empty location node while parsing %s", filename);
 						bail = TRUE;
 						xmlFree (type);
@@ -1588,16 +1771,32 @@ nautilus_application_load_session (NautilusApplication *application,
 							gtk_window_unmaximize (GTK_WINDOW (window));
 						}
 
+						if (xmlHasProp (node, "sticky")) {
+							gtk_window_stick (GTK_WINDOW (window));
+						} else {
+							gtk_window_unstick (GTK_WINDOW (window));
+						}
+
+						if (xmlHasProp (node, "keep-above")) {
+							gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);
+						} else {
+							gtk_window_set_keep_above (GTK_WINDOW (window), FALSE);
+						}
+
+						location = g_file_new_for_uri (location_uri);
 						nautilus_window_open_location (window, location, FALSE);
+						g_object_unref (location);
 					} else if (!strcmp (type, "spatial")) {
+						location = g_file_new_for_uri (location_uri);
 						window = nautilus_application_present_spatial_window (application, NULL, NULL, location, gdk_screen_get_default ());
+						g_object_unref (location);
 					} else {
 						g_message ("unknown window type \"%s\" while parsing %s", type, filename);
 						bail = TRUE;
 					}
 
 					xmlFree (type);
-					xmlFree (location);
+					xmlFree (location_uri);
 				} else {
 					g_message ("unexpected node %s while parsing %s", node->name, filename);
 					bail = TRUE;
@@ -1614,25 +1813,16 @@ nautilus_application_load_session (NautilusApplication *application,
 	if (bail) {
 		g_message ("failed to load session from %s", filename);
 	} else {
-		char *uri;
-		GnomeVFSFileInfo *info;
+		struct stat buf;
+		
 		/* only remove file if it is regular, user-owned and the user has write access. */
 
-		uri = gnome_vfs_get_uri_from_local_path (filename);
-		info = gnome_vfs_file_info_new ();
-		if (uri != NULL &&
-		    gnome_vfs_get_file_info (uri, info, GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK &&
-		    (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE) &&
-		    (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS) &&
-		    (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_IDS) &&
-		    info->type == GNOME_VFS_FILE_TYPE_REGULAR &&
-		    (info->permissions & (GNOME_VFS_PERM_USER_WRITE |
-					  GNOME_VFS_PERM_USER_WRITE)) &&
-		    info->uid == geteuid ()) {
+		if (g_stat (filename, &buf) == 0 &&
+		    S_ISREG (buf.st_mode) &&
+		    (buf.st_mode & S_IWUSR) &&
+		    buf.st_uid == geteuid()) {	
 			g_remove (filename);
 		}
-		gnome_vfs_file_info_unref (info);
-		g_free (uri);
 	}
 }
 
